from adari_core.plots.combined import CombinedPlot
from adari_core.plots.cut import CutPlot
from adari_core.plots.panel import Panel
from adari_core.plots.histogram import HistogramPlot
from adari_core.plots.images import ImagePlot, CentralImagePlot
from adari_core.plots.points import ScatterPlot
from adari_core.plots.text import TextPlot
from adari_core.report import AdariReportBase
from adari_core.utils.utils import fetch_kw_or_default

from .sphere_utils import SphereSetupInfo
import astropy.io.fits as fits
import os
import numpy as np

from . import SphereReportMixin


class SphereStdStarReport(SphereReportMixin, AdariReportBase):

    center_size = 200
    hist_bins_max = 50

    def __init__(self):
        super().__init__("sphere_std_star")

    def parse_sof(self):

        isi_product = None
        isi_raw = None
        
        for filename, catg in self.inputs:
            if catg == "IRD_SCIENCE_IMAGING":
                isi_product = filename
            if catg == "IRD_SCIENCE_IMAGING_RAW":
                isi_raw = filename
        
        file_lists = []
        if isi_product is not None and isi_raw is not None:
            file_lists.append(
                {
                    "master_im": isi_product,
                    "raw_im": isi_raw,
                }
            )

        return file_lists

    def generate_panels(self, **kwargs):
        """Create panels."""

        """Create panels."""
        panels = {}
        p = Panel(4, 3, height_ratios=[1, 4, 4])


                # Metadata in Text Plot
        px, py = 0, 0
        vspace = 0.3
        t1 = TextPlot(columns=1, v_space=vspace)
        master_im = self.hdus[0]["master_im"]
        fname = os.path.basename(str(master_im.filename()))
        raw_im = self.hdus[0]["raw_im"]

        col1 = (
            str(
                master_im["PRIMARY"].header.get(
                    "INSTRUME", "Missing INSTRUME"
                )
            ),
            "PRO CATG: "
            + str(
                master_im["PRIMARY"].header.get(
                    "HIERARCH ESO PRO CATG", "Missing PRO CATG"
                )
            ),
            "MASTER FILE NAME: " + fname,
            "RAW1 NAME: "
            + str(
                master_im["PRIMARY"].header.get(
                    "HIERARCH ESO PRO REC1 RAW1 NAME", "Missing RAW1 NAME"
                )
            ),
        )
        t1.add_data(col1)
        p.assign_plot(t1, px, py, xext=2)

        px = px + 2
        t2 = TextPlot(columns=1, v_space=vspace, xext=1)
        self.metadata = SphereSetupInfo.ird_science_imaging(master_im)
        t2.add_data(self.metadata)
        p.assign_plot(t2, px, py, xext=1)


        # Full reduced image
        vspace = 0.1
        fullfield = ImagePlot(title="Reduced field (full)", v_clip="percentile",
                          v_clip_kwargs={"percentile": 95.0})
        fullfield.add_data(master_im["IMAGE"].data)
        p.assign_plot(fullfield, 0, 1, xext=1, yext=1)

        # Central region of reduced image, with found standards marked
        centerfield = CentralImagePlot(
            fullfield,
            title="Reduced field (central {} pixels)".format(self.center_size),
            v_clip="percentile",
            v_clip_kwargs={"percentile": 95.0}
        )

        p.assign_plot(centerfield, 0, 2)

        # Cut plot X
        cutX = CutPlot("x", title="Central column", x_label="y")
        cutX.y_label = fetch_kw_or_default(master_im["IMAGE"], "BUNIT", default="ADU")
        cutX.add_data(
            fullfield,
            fullfield.get_data_coord(fullfield.data.shape[1] // 2, "x"),
            label="Full",
            color="red",
        )
        p.assign_plot(cutX, 1, 1, xext=1, yext=1)

        # Cut plot Y
        cutY = CutPlot("y", title="Central row", x_label="x")
        cutY.y_label = fetch_kw_or_default(master_im["IMAGE"], "BUNIT", default="ADU")
        cutY.add_data(
            fullfield,
            fullfield.get_data_coord(fullfield.data.shape[0] // 2, "y"),
            label="Full",
            color="red",
        )
        p.assign_plot(cutY, 2, 1, xext=1, yext=1)

        # Do same for the central region
        # Cut plot X
        cutX_cent = CutPlot("x", title="Central region (Y)", x_label="y")
        cutX_cent.y_label = fetch_kw_or_default(
            master_im["IMAGE"], "BUNIT", default="ADU"
        )
        cutX_cent.add_data(
            centerfield,
            centerfield.get_data_coord(centerfield.data.shape[1] // 2, "x"),
            label="Central",
            color="red",
        )
        p.assign_plot(cutX_cent, 1, 2, xext=1, yext=1)

        # Cut plot Y
        cutY_cent = CutPlot("y", title="Central region (X)", x_label="x")
        cutY_cent.y_label = fetch_kw_or_default(
            master_im["IMAGE"], "BUNIT", default="ADU"
        )
        cutY_cent.add_data(
            centerfield,
            centerfield.get_data_coord(centerfield.data.shape[0] // 2, "y"),
            label="Central",
            color="red",
        )
        p.assign_plot(cutY_cent, 2, 2, xext=1, yext=1)

        # Plot histogram
        # scaling = {}
        # scaling["v_clip"] = "sigma"
        # scaling["v_clip_kwargs"] = {"nsigma": 5}

        histogram = HistogramPlot(
            master_data=master_im["IMAGE"].data,
            raw_data=raw_im["PRIMARY"].data if raw_im is not None else None,
            bins=self.hist_bins_max,
            title="Counts histogram",
            master_label="reduced",
            #**scaling,
        )
        if raw_im is not None:
            bin_per_val = self.hist_bins_max + 1  # Dummy val to start while
            vals_per_bin = 1
            rmin, rmax = histogram.get_vlim()
            while bin_per_val > self.hist_bins_max:
                bin_per_val = (rmax - rmin + 1) // vals_per_bin
                vals_per_bin += 1
            histogram.bins = bin_per_val
        p.assign_plot(histogram, 3, 1, xext=1, yext=1)

        # Metadata
        panels[p] = {
            "report_name": "SPHERE_std_star_ext_{}".format("IMAGE"),
            "report_description": "StandardStar_{}".format("IMAGE"),
            "report_tags": [],
        }

        return panels


rep = SphereStdStarReport()
